/*
 * Copyright (C) 1999 Colin Van Dyke
 * Copyright (C) 1999-2002 Vojtech Pavlik
 * Copyright (C) 2006 Jean-Yves Lefort <jylefort@FreeBSD.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _LINUX_JS_H
#define _LINUX_JS_H

#include <sys/cdefs.h>
#include <sys/types.h>
#include <sys/conf.h>
#include <sys/selinfo.h>
#include <sys/callout.h>
#include <sys/bitstring.h>
#include <sys/queue.h>
#include "joystick.h"
#include "linux_js_queue.h"

/*
 * Definitions from linux/input.h
 */

#define EV_KEY			0x01
#define EV_ABS			0x03

#define BTN_MOUSE		0x110

#define BTN_JOYSTICK		0x120
#define BTN_TRIGGER		0x120
#define BTN_THUMB		0x121
#define BTN_DEAD		0x12f

#define BTN_GAMEPAD		0x130
#define BTN_SELECT		0x13a
#define BTN_START		0x13b

#define BTN_TOOL_PEN		0x140
#define BTN_TOOL_RUBBER		0x141
#define BTN_TOOL_FINGER		0x145
#define BTN_TOOL_MOUSE		0x146
#define BTN_STYLUS		0x14b

#define ABS_X			0x00
#define ABS_Y			0x01
#define ABS_THROTTLE		0x06
#define ABS_RUDDER		0x07
#define ABS_HAT0X		0x10
#define ABS_PRESSURE		0x18
#define ABS_MISC		0x28

#define LINUX_JS(instance)		((struct linux_js *) instance)
#define LINUX_JS_CLASS(class)		((struct linux_js_class *) class)

struct linux_js;

struct linux_js_class
{
  unsigned int		instance_size;

  int	(*open)		(struct linux_js *js);
  int	(*close)	(struct linux_js *js);
  void	(*read)		(struct linux_js *js);
  void	(*destroy)	(struct linux_js *js);
};

struct linux_js
{
  struct linux_js_class		*class;

  struct cdev			*dev;
  char				*name;
  struct linux_js_queue		queue;
  struct selinfo		sel;
  struct callout		callout;

  SLIST_ENTRY(linux_js)		next;

  /* in Linux the following members belong to struct joydev */

  int				nkey;
  int				nabs;

  uint16_t			keymap[KEY_MAX - BTN_MISC + 1];
  uint16_t			keypam[KEY_MAX - BTN_MISC + 1];

  uint8_t			absmap[ABS_MAX + 1];
  uint8_t			abspam[ABS_MAX + 1];

  struct js_corr		corr[ABS_MAX + 1];

  int16_t			jabs[ABS_MAX + 1];

  /* in Linux the following members belong to struct input_dev */

  bitstr_t			bit_decl(keybit, KEY_MAX + 1);
  bitstr_t			bit_decl(absbit, ABS_MAX + 1);

  int				absmax[ABS_MAX + 1];
  int				absmin[ABS_MAX + 1];
  int				absfuzz[ABS_MAX + 1];
  int				absflat[ABS_MAX + 1];

  bitstr_t			bit_decl(key, KEY_MAX + 1);
  int				abs[ABS_MAX + 1];
};

struct linux_js *linux_js_new (struct linux_js_class *class);
void linux_js_free (struct linux_js *js);

int linux_js_attach (struct linux_js *js, const char *native_devname);

void linux_js_set_name (struct linux_js *js, const char *name);
void linux_js_take_name (struct linux_js *js, char *name);

void linux_js_set_abs_params (struct linux_js *js, int code, int min, int max, int fuzz, int flat);

void linux_js_event (struct linux_js *js,
		     int initial_state,
		     unsigned int type,
		     unsigned int code,
		     int value);

#endif /* _LINUX_JS_H */
