/*
 * Copyright (C) 2006 Jean-Yves Lefort <jylefort@FreeBSD.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <sys/cdefs.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/sysproto.h>
#include <sys/syscallsubr.h>
#include <sys/fcntl.h>
#include "linux_js_devglue.h"
#include "linux_js_util.h"

static int
linux_js_devglue_open (struct linux_js *js)
{
  struct linux_js_devglue *devglue = LINUX_JS_DEVGLUE(js);
  int status;

  if (devglue->fd != -1)
    return EBUSY;

  status = kern_open(curthread, devglue->filename, UIO_SYSSPACE, O_RDONLY | O_NONBLOCK, 0);
  if (status == 0)
    {
      struct linux_js_devglue_class *class = LINUX_JS_DEVGLUE_CLASS(js->class);

      devglue->fd = curthread->td_retval[0];

      if (class->open)
	class->open(devglue);
    }

  return status;
}

static int
linux_js_devglue_close (struct linux_js *js)
{
  struct linux_js_devglue *devglue = LINUX_JS_DEVGLUE(js);
  int status;

  status = linux_js_kern_close(devglue->fd);
  if (status == 0)
    devglue->fd = -1;

  return status;
}

void
linux_js_devglue_class_init (struct linux_js_devglue_class *class)
{
  struct linux_js_class *js_class = LINUX_JS_CLASS(class);
  int i;

  LINUX_JS_ASSERT(class != NULL);
  LINUX_JS_ASSERT(class->attach != NULL);

  if (js_class->instance_size == 0)
    js_class->instance_size = sizeof(struct linux_js_devglue);

  js_class->open = linux_js_devglue_open;
  js_class->close = linux_js_devglue_close;

  for (i = 0;; i++)
    {
      char filename[PATH_MAX];

      sprintf(filename, "/dev/%s%d", class->devname, i);
      if (linux_js_exists(filename))
	{
	  struct linux_js *js;
	  struct linux_js_devglue *devglue;

	  js = linux_js_new(js_class);
	  devglue = LINUX_JS_DEVGLUE(js);
	  strcpy(devglue->filename, filename);
	  devglue->devname = filename + 5;
	  devglue->fd = -1;

	  if (! class->attach(devglue) || ! linux_js_attach(js, filename))
	    linux_js_free(js);
	}
      else
	break;
    }
}
