/*
 * Copyright (C) 2006 Jean-Yves Lefort <jylefort@FreeBSD.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <sys/cdefs.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/types.h>
#include <sys/joystick.h>
#include "linux_js_joy.h"
#include "linux_js_devglue.h"
#include "linux_js_util.h"

#define INVALID_AXIS			0x80000000 /* sys/dev/joy/joy.c */

static struct linux_js_devglue_class class;

static int
linux_js_joy_attach (struct linux_js_devglue *devglue)
{
  struct linux_js *js = LINUX_JS(devglue);
  char name[64];

  bit_set(js->absbit, ABS_X);
  bit_set(js->absbit, ABS_Y);
  bit_set(js->keybit, BTN_TRIGGER);
  bit_set(js->keybit, BTN_THUMB);

  sprintf(name, "PC joystick %s", devglue->devname);
  linux_js_set_name(js, name);

  return 1;
}

static void
linux_js_joy_report (struct linux_js_devglue *devglue, int initial_state)
{
  struct linux_js *js = LINUX_JS(devglue);
  struct joystick data;

  LINUX_JS_ASSERT(devglue != NULL);

  if (linux_js_kern_read(devglue->fd, &data, sizeof(data)) == sizeof(data))
    {
      if (data.x != INVALID_AXIS)
	linux_js_event(js, initial_state, EV_ABS, ABS_X, data.x);
      if (data.y != INVALID_AXIS)
	linux_js_event(js, initial_state, EV_ABS, ABS_Y, data.y);

      linux_js_event(js, initial_state, EV_KEY, BTN_TRIGGER, data.b1);
      linux_js_event(js, initial_state, EV_KEY, BTN_THUMB, data.b2);
    }
}

static void
linux_js_joy_open (struct linux_js_devglue *devglue)
{
  linux_js_joy_report(devglue, 1);
}

static void
linux_js_joy_read (struct linux_js *js)
{
  linux_js_joy_report(LINUX_JS_DEVGLUE(js), 0);
}

void
linux_js_joy_class_init (void)
{
  struct linux_js_class *js_class = LINUX_JS_CLASS(&class);

  bzero(&class, sizeof(class));

  js_class->read = linux_js_joy_read;

  class.devname = "joy";
  class.attach = linux_js_joy_attach;
  class.open = linux_js_joy_open;

  linux_js_devglue_class_init(&class);
}
