/*
 * Copyright (C) 2006 Jean-Yves Lefort <jylefort@FreeBSD.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/sockio.h>
#include <sys/sysproto.h>
#include <i386/linux/linux.h>
#include <i386/linux/linux_proto.h>
#include <compat/linux/linux_ioctl.h>
#include "joystick.h"

/* ioctl macros from the Linux kernel sources */

#define _IOC_NRBITS	8
#define _IOC_TYPEBITS	8
#define _IOC_SIZEBITS	14

#define _IOC_SIZEMASK	((1 << _IOC_SIZEBITS)-1)

#define _IOC_NRSHIFT	0
#define _IOC_TYPESHIFT	(_IOC_NRSHIFT+_IOC_NRBITS)
#define _IOC_SIZESHIFT	(_IOC_TYPESHIFT+_IOC_TYPEBITS)

#define _IOC_SIZE(nr)	(((nr) >> _IOC_SIZESHIFT) & _IOC_SIZEMASK)

#define LINUX_JSIOCGVERSION	0x6A01

#define LINUX_JSIOCGAXES	0x6A11
#define LINUX_JSIOCGBUTTONS	0x6A12
#define LINUX_JSIOCGNAME	0x6A13

#define LINUX_JSIOCSCORR	0x6A21
#define LINUX_JSIOCGCORR	0x6A22

#define LINUX_JSIOCSAXMAP	0x6A31
#define LINUX_JSIOCGAXMAP	0x6A32
#define LINUX_JSIOCSBTNMAP	0x6A33
#define LINUX_JSIOCGBTNMAP	0x6A34

static int
linux_js_linux_ioctl (struct thread *td, struct linux_ioctl_args *args)
{
  switch (args->cmd & 0xFFFF)
    {
    case LINUX_JSIOCGVERSION:
      args->cmd = JSIOCGVERSION;
      break;

    case LINUX_JSIOCGAXES:
      args->cmd = JSIOCGAXES;
      break;

    case LINUX_JSIOCGBUTTONS:
      args->cmd = JSIOCGBUTTONS;
      break;

    case LINUX_JSIOCGNAME:
      args->cmd = JSIOCGNAME(_IOC_SIZE(args->cmd));
      break;

    case LINUX_JSIOCSCORR:
      args->cmd = JSIOCSCORR;
      break;

    case LINUX_JSIOCGCORR:
      args->cmd = JSIOCGCORR;
      break;

    case LINUX_JSIOCSAXMAP:
      args->cmd = JSIOCSAXMAP;
      break;

    case LINUX_JSIOCGAXMAP:
      args->cmd = JSIOCGAXMAP;
      break;

    case LINUX_JSIOCSBTNMAP:
      args->cmd = JSIOCSBTNMAP;
      break;

    case LINUX_JSIOCGBTNMAP:
      args->cmd = JSIOCGBTNMAP;
      break;

    default:
      return ENOIOCTL;
    }

  return ioctl(td, (struct ioctl_args *) args);
}

static struct linux_ioctl_handler linux_js_ioctl_handler =
  { linux_js_linux_ioctl, 0x6A01, 0x6A34 };

SYSINIT(linux_js_ioctl_register, SI_SUB_KLD, SI_ORDER_MIDDLE, linux_ioctl_register_handler, &linux_js_ioctl_handler);
SYSUNINIT(linux_js_ioctl_unregister, SI_SUB_KLD, SI_ORDER_MIDDLE, linux_ioctl_unregister_handler, &linux_js_ioctl_handler);
