/*
 * Copyright (C) 2006 Jean-Yves Lefort <jylefort@FreeBSD.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _LINUX_JS_QUEUE_H
#define _LINUX_JS_QUEUE_H

#include <sys/cdefs.h>
#include <sys/types.h>
#include <sys/queue.h>

struct linux_js_queue_chunk
{
  off_t					offset;
  size_t				size;
  STAILQ_ENTRY(linux_js_queue_chunk)	next;
  char					data[];
};

STAILQ_HEAD(linux_js_queue_chunks, linux_js_queue_chunk);

struct linux_js_queue
{
  size_t				size;
  size_t				chunk_size;
  struct linux_js_queue_chunks		chunks;
};

void linux_js_queue_init (struct linux_js_queue *queue, size_t chunk_size);
void linux_js_queue_clear (struct linux_js_queue *queue);

void linux_js_queue_push_tail (struct linux_js_queue *queue, const void *data, size_t size);
void linux_js_queue_pop (struct linux_js_queue *queue, void *data, size_t size);

#endif /* _LINUX_JS_QUEUE_H */
