/*
 * Copyright (C) 2006 Jean-Yves Lefort <jylefort@FreeBSD.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _LINUX_JS_UTIL_H
#define _LINUX_JS_UTIL_H

#include <sys/cdefs.h>
#include <sys/types.h>
#include <sys/bitstring.h>
#include <sys/proc.h>

#ifdef WITH_DEBUG
#define LINUX_JS_ASSERT(exp) do {			\
    if (__predict_false(! (exp)))			\
      panic("%s:%i (%s): assertion failed: (%s)",	\
	    __FILE__,					\
	    __LINE__,					\
	    __func__,					\
	    #exp);					\
  } while (0)
#define LINUX_JS_ASSERT_NOT_REACHED() do {		\
    panic("%s:%i (%s): should not be reached",		\
	  __FILE__,					\
	  __LINE__,					\
	  __func__);					\
  } while (0)
#else
#define LINUX_JS_ASSERT(exp)
#define LINUX_JS_ASSERT_NOT_REACHED()
#endif

int linux_js_exists (const char *filename);

ssize_t linux_js_kern_read (int fd, void *buf, size_t nbytes);
int linux_js_kern_ioctl (int fd, u_long com, void *data);
int linux_js_kern_close (int fd);

int linux_js_bit_test_and_set (bitstr_t *name, int bit);
int linux_js_bit_find_next_zero (bitstr_t *name, int size, int offset);

#endif /* _LINUX_JS_UTIL_H */
